==> rlwrap-0.18.tar.gz: rlwrap-0.18/src/term.c <==
/*  term.c: terminal handling, cursor movement etc. */

/*  This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License , or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  hlub@knoware.nl
*/


#include "rlwrap.h"

/*global vars */
char term_eof;          /* end_of_file char */
char term_stop;         /* stop (suspend) key */
char *term_backspace;   /* backspace control seq  (or 0, if none defined in terminfo) */
int redisplay = 1;

struct termios saved_terminal_settings; /* original terminal settings */
int terminal_settings_saved = FALSE;    /*  saved_terminal_settings is valid */
struct winsize window_size;             /* current window size */

static char *term_cr;          /* carriage return (or 0, if none defined in terminfo) */
static char * term_clear_line; 



void 
init_terminal (void)        /* save term settings and determine term type */ 
{
  char *term_name;
  char *term_buf;
  char *term_string_buf; 
  
  if (! isatty (STDIN_FILENO))
    myerror("stdin is not a tty");
  if (tcgetattr(STDIN_FILENO, &saved_terminal_settings) < 0)
    myerror ("tcgetattr error on stdin");
  else
    terminal_settings_saved = TRUE;
  if (ioctl (STDIN_FILENO, TIOCGWINSZ, &window_size) < 0)
    myerror("Could not get winsize");
  /* unsetenv("TERM");    */ 
  
  /* init some variables: */
  term_name = getenv ("TERM");

  
  if (!term_name)
    term_name = "dumb";
  term_buf = (char*) mymalloc (4096);
  term_string_buf = (char *) mymalloc(2048);

  
  term_backspace = NULL;
  term_cr        = NULL;
  term_clear_line = NULL;

#ifdef HAVE_TERM_H
  if (tgetent (term_buf, term_name) > 0) {
    term_backspace  = tgetstr ("le", &term_string_buf);
    term_cr         = tgetstr ("cr", &term_string_buf);
    term_clear_line = tgetstr ("dl1", &term_string_buf);
  }
#endif  
  
  term_eof = saved_terminal_settings.c_cc[VEOF];
  term_stop = saved_terminal_settings.c_cc[VSTOP];

  DPRINTF2(DEBUG_TERMIO,"term_eof=%d term_stop=%d", term_eof, term_stop);
}


void set_echo(int yes)
{
  
  struct termios pterm = saved_terminal_settings;
  pterm.c_lflag &= ~ICANON;
  pterm.c_lflag &= ~ECHO;
  pterm.c_lflag |= ISIG;
  pterm.c_cc[VMIN] = 1;
  pterm.c_cc[VTIME]=0;
  if (yes)
    pterm.c_lflag |= ECHO;
  else /* no */
    pterm.c_lflag &= ~ECHO;
  if (tcsetattr(STDIN_FILENO, TCSANOW, &pterm) < 0 && errno != ENOTTY)
    ; /* myerror ("cannot prepare terminal (tcsetattr error on stdin)"); */
}


#ifndef HAVE_TERM_H
#define tputs(a,b,c) 
#endif

void 
backspace (int count) 
{
  int i;
  if (term_backspace)
    for (i = 0; i < count; i++)
      tputs (term_backspace, 1, my_putchar);
  else
    for (i = 0; i < count; i++)
      my_putchar ('\b');
}

void 
cr () 
{
  if (term_cr)
    tputs (term_cr, 1, my_putchar);
  else
    my_putchar('\r');
}

void 
clear_line() 
{
  int i;
  int width = window_size.ws_col;
  char *p, *spaces;
  cr();
  if (term_clear_line)
    tputs (term_clear_line, 1, my_putchar);
  else {
    spaces = (char *) mymalloc(width);
    for (i=0, p = spaces; i< width; i++, p++)
      *p = ' ';
    write(STDOUT_FILENO, spaces, width);
    free((void *) spaces);
  }
  cr();
}


int 
my_putchar 
(int c) {
  char ch = c;
  ssize_t nwritten; 
  nwritten = write(STDOUT_FILENO, &ch, 1);
  return (nwritten == -1 ? -1 : c);
}

==> rlwrap-0.21.tar.gz: rlwrap-0.21/src/term.c <==
/*  term.c: terminal handling, cursor movement etc. */

/*  This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License , or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

    You may contact the author by:
       e-mail:  hlub@knoware.nl
*/


#include "rlwrap.h"

/*global vars */
char term_eof;          /* end_of_file char */
char term_stop;         /* stop (suspend) key */
char *term_backspace;   /* backspace control seq  (or 0, if none defined in terminfo) */
char *term_cursor_hpos;
char *term_clear_screen;
char *term_cursor_up;
char *term_cursor_down;

int redisplay = 1;

struct termios saved_terminal_settings; /* original terminal settings */
int terminal_settings_saved = FALSE;    /*  saved_terminal_settings is valid */
struct winsize window_size;             /* current window size */

static char *term_cr;          /* carriage return (or 0, if none defined in terminfo) */
static char *term_clear_line; 


void 
init_terminal (void)        /* save term settings and determine term type */ 
{
  char *term_name;
  char *term_buf;
  char *term_string_buf; 
  
  if (! isatty (STDIN_FILENO))
    myerror("stdin is not a tty");
  if (tcgetattr(STDIN_FILENO, &saved_terminal_settings) < 0)
    myerror ("tcgetattr error on stdin");
  else
    terminal_settings_saved = TRUE;
  if (ioctl (STDIN_FILENO, TIOCGWINSZ, &window_size) < 0)
    myerror("Could not get winsize");
  /* unsetenv("TERM");    */ 
  
  /* init some variables: */
  term_name = getenv ("TERM");

  
  if (!term_name)
    term_name = "dumb";
  term_buf = (char*) mymalloc (4096);
  term_string_buf = (char *) mymalloc(2048);

  
  term_backspace = NULL;
  term_cr = NULL;
  term_clear_line = NULL;
  term_cursor_hpos = NULL;

#ifdef HAVE_TERM_H
  if (tgetent (term_buf, term_name) > 0) {
    term_backspace    = tgetstr("le", &term_string_buf);
    term_cr           = tgetstr("cr", &term_string_buf);
    term_clear_line   = tgetstr("dl1", &term_string_buf);
    term_clear_screen = tgetstr("cl", &term_string_buf);
    term_cursor_hpos  = tgetstr("ch", &term_string_buf);
    term_cursor_up    = tgetstr("up", &term_string_buf);
    term_cursor_down  = tgetstr("do", &term_string_buf);
  }
#endif  
  
  term_eof = saved_terminal_settings.c_cc[VEOF];
  term_stop = saved_terminal_settings.c_cc[VSTOP];

  DPRINTF3(DEBUG_TERMIO,"term_eof=%d term_cursor_up=%s term_cursor_hpos=%s", term_eof,term_cursor_up ,term_cursor_hpos);
}


void set_echo(int yes)
{
  
  struct termios pterm = saved_terminal_settings;
  pterm.c_lflag &= ~ICANON;
  pterm.c_lflag &= ~ECHO;
  pterm.c_lflag |= ISIG;
  pterm.c_cc[VMIN] = 1;
  pterm.c_cc[VTIME]=0;
  if (yes)
    pterm.c_lflag |= ECHO;
  else /* no */
    pterm.c_lflag &= ~ECHO;
  if (tcsetattr(STDIN_FILENO, TCSANOW, &pterm) < 0 && errno != ENOTTY)
    ; /* myerror ("cannot prepare terminal (tcsetattr error on stdin)"); */
}


#ifndef HAVE_TERM_H
#define tputs(a,b,c) 
#endif

void 
backspace (int count) 
{
  int i;
  if (term_backspace)
    for (i = 0; i < count; i++)
      tputs (term_backspace, 1, my_putchar);
  else
    for (i = 0; i < count; i++)
      my_putchar ('\b');
}

void
cursor_hpos (int col)
{
  assert(term_cursor_hpos != NULL); /* caller has to make sure */
  tputs(tgoto(term_cursor_hpos,0,col), 1,  my_putchar);
}
  
void 
cr () 
{
  if (term_cr)
    tputs (term_cr, 1, my_putchar);
  else
    my_putchar('\r');
}

void
clear_the_screen() /* clear_screen is a macro in term.h */
{
  int i;
  if (term_clear_screen)
    tputs (term_clear_screen, 1, my_putchar); 
  else
    for (i = 0; i < window_size.ws_row; i++)
      my_putchar('\n');  /* poor mans clear screen */
}

void 
clear_line() 
{
  int i;
  int width = window_size.ws_col;
  char *p, *spaces;
  cr();
  if (term_clear_line)
    tputs (term_clear_line, 1, my_putchar);
  else {
    spaces = (char *) mymalloc(width);
    for (i=0, p = spaces; i< width; i++, p++)
      *p = ' ';
    write(STDOUT_FILENO, spaces, width);  /* poor mans clear line */
    free((void *) spaces);
  }
  cr();
}


void
curs_up()
{
  assert(term_cursor_up != NULL); /* caller has to make sure */
  tputs(term_cursor_up, 1,  my_putchar);
}
	 
void
curs_down()
{
  assert(term_cursor_down != NULL); /* caller has to make sure */
  tputs(term_cursor_down, 1,  my_putchar);
}

int 
my_putchar 
(int c) {
  char ch = c;
  ssize_t nwritten; 
  nwritten = write(STDOUT_FILENO, &ch, 1);
  return (nwritten == -1 ? -1 : c);
}
